import { ControllerRoute } from '@app/common/constants/controller-route';
import { EnableDisableStatusEnum } from '@app/common/constants/days.enum';
import { BaseResponseDto } from '@app/common/dto/base.response.dto';
import { JwtAuthGuard } from '@app/common/guards/jwt.auth.guard';
import {
  Body,
  Controller,
  Get,
  Param,
  Post,
  Put,
  Query,
  Req,
  UseGuards,
} from '@nestjs/common';
import { ApiBearerAuth, ApiOperation, ApiTags } from '@nestjs/swagger';

import { PageResponse } from '@app/common/dto/pagination.response.dto';
import { SuccessResponseDto } from '@app/common/dto/success.response.dto';
import { plainToInstance } from 'class-transformer';
import { CreateBookableSpaceDto } from '../dtos';
import { BookableSpaceRequestDto } from '../dtos/bookable-space-request.dto';
import { BookableSpaceResponseDto } from '../dtos/bookable-space-response.dto';
import { UpdateBookableSpaceDto } from '../dtos/update-bookable-space.dto';
import { BookableSpaceService } from '../services';

@ApiTags('Booking Module')
@Controller({
  version: EnableDisableStatusEnum.ENABLED,
  path: ControllerRoute.BOOKABLE_SPACES.ROUTE,
})
export class BookableSpaceController {
  constructor(private readonly bookableSpaceService: BookableSpaceService) {}
  @ApiBearerAuth()
  @UseGuards(JwtAuthGuard)
  @Post()
  @ApiOperation({
    summary:
      ControllerRoute.BOOKABLE_SPACES.ACTIONS.ADD_BOOKABLE_SPACES_SUMMARY,
    description:
      ControllerRoute.BOOKABLE_SPACES.ACTIONS.ADD_BOOKABLE_SPACES_DESCRIPTION,
  })
  async create(@Body() dto: CreateBookableSpaceDto): Promise<BaseResponseDto> {
    const result = await this.bookableSpaceService.create(dto);
    return new SuccessResponseDto({
      data: result,
      message: 'Successfully created bookable spaces',
    });
  }

  @ApiBearerAuth()
  @UseGuards(JwtAuthGuard)
  @Get()
  @ApiOperation({
    summary:
      ControllerRoute.BOOKABLE_SPACES.ACTIONS.GET_ALL_BOOKABLE_SPACES_SUMMARY,
    description:
      ControllerRoute.BOOKABLE_SPACES.ACTIONS
        .GET_ALL_BOOKABLE_SPACES_DESCRIPTION,
  })
  async findAll(
    @Query() query: BookableSpaceRequestDto,
    @Req() req: Request,
  ): Promise<PageResponse<BookableSpaceResponseDto>> {
    const project = req['user']?.project?.uuid;
    if (!project) {
      throw new Error('Project UUID is required in the request');
    }
    const { data, pagination } = await this.bookableSpaceService.findAll(
      query,
      project,
    );
    return new PageResponse<BookableSpaceResponseDto>(
      {
        data: data.map((space) =>
          plainToInstance(BookableSpaceResponseDto, space, {
            excludeExtraneousValues: true,
          }),
        ),
        message: 'Successfully fetched all bookable spaces',
      },
      pagination,
    );
  }

  @ApiBearerAuth()
  @UseGuards(JwtAuthGuard)
  @Put(':spaceUuid')
  @ApiOperation({
    summary:
      ControllerRoute.BOOKABLE_SPACES.ACTIONS.UPDATE_BOOKABLE_SPACES_SUMMARY,
    description:
      ControllerRoute.BOOKABLE_SPACES.ACTIONS
        .UPDATE_BOOKABLE_SPACES_DESCRIPTION,
  })
  async update(
    @Param('spaceUuid') spaceUuid: string,
    @Body() dto: UpdateBookableSpaceDto,
  ): Promise<BaseResponseDto> {
    const result = await this.bookableSpaceService.update(spaceUuid, dto);
    return new SuccessResponseDto({
      data: result,
      message: 'Successfully updated bookable spaces',
    });
  }
}
