import {
  BadRequestException,
  HttpException,
  HttpStatus,
  Injectable,
} from '@nestjs/common';
import {
  GetPowerClampBySpaceDto,
  GetPowerClampDto,
} from '../dto/get-power-clamp.dto';
import {
  PowerClampDailyRepository,
  PowerClampHourlyRepository,
  PowerClampMonthlyRepository,
} from '@app/common/modules/power-clamp/repositories';
import { SuccessResponseDto } from '@app/common/dto/success.response.dto';
import {
  PowerClampParamsDto,
  ResourceParamsDto,
} from '../dto/power-clamp-params.dto';
import { DeviceRepository } from '@app/common/modules/device/repositories';
import { SpaceDeviceService } from 'src/space/services';
import { SqlLoaderService } from '@app/common/helper/services/sql-loader.service';
import { DataSource } from 'typeorm';
import { SQL_PROCEDURES_PATH } from '@app/common/constants/sql-query-path';
import { ProductType } from '@app/common/constants/product-type.enum';
import { CommunityService } from 'src/community/services';
import { BaseResponseDto } from '@app/common/dto/base.response.dto';
import { filterByMonth, toMMYYYY } from '@app/common/helper/date-format';

@Injectable()
export class PowerClampService {
  constructor(
    private readonly powerClampDailyRepository: PowerClampDailyRepository,
    private readonly powerClampHourlyRepository: PowerClampHourlyRepository,
    private readonly powerClampMonthlyRepository: PowerClampMonthlyRepository,
    private readonly deviceRepository: DeviceRepository,
    private readonly spaceDeviceService: SpaceDeviceService,
    private readonly sqlLoader: SqlLoaderService,
    private readonly dataSource: DataSource,
    private readonly communityService: CommunityService,
  ) {}

  async getPowerClampDataBySpaceOrCommunity(
    params: ResourceParamsDto,
    query: GetPowerClampBySpaceDto,
  ): Promise<BaseResponseDto> {
    const { monthDate, groupByDevice } = query;
    const { spaceUuid, communityUuid } = params;

    try {
      // Validate we have at least one identifier
      if (!spaceUuid && !communityUuid) {
        throw new BadRequestException(
          'Either spaceUuid or communityUuid must be provided',
        );
      }

      // Get devices based on space or community
      const devices = spaceUuid
        ? await this.spaceDeviceService.getAllDevicesBySpace(spaceUuid)
        : await this.communityService.getAllDevicesByCommunity(communityUuid);

      if (!devices?.length) {
        return this.buildResponse(
          `No devices found for ${spaceUuid ? 'space' : 'community'}`,
          [],
        );
      }

      // Filter power clamp devices
      const powerClampDevices = devices.filter(
        (device) => device.productDevice?.prodType === ProductType.PC,
      );

      if (powerClampDevices.length === 0) {
        return this.buildResponse(
          `No power clamp devices found for ${spaceUuid ? 'space' : 'community'}`,
          [],
        );
      }

      const formattedMonthDate = toMMYYYY(monthDate);

      if (groupByDevice) {
        // Handle per-device response
        const deviceDataPromises = powerClampDevices.map(async (device) => {
          const data = await this.executeProcedure(
            'fact_daily_space_energy_consumed_procedure',
            [formattedMonthDate, device.uuid],
          );

          const formattedData = data.map((item) => ({
            ...item,
            date: new Date(item.date).toLocaleDateString('en-CA'), // YYYY-MM-DD
          }));

          const filteredData = monthDate
            ? filterByMonth(formattedData, monthDate)
            : formattedData;

          return {
            deviceUuid: device.uuid,
            deviceName: device.name || `Power Clamp Device`,
            data: filteredData,
          };
        });

        const deviceDataResults = await Promise.all(deviceDataPromises);

        return this.buildResponse(
          `Power clamp data fetched successfully for ${spaceUuid ? 'space' : 'community'}`,
          deviceDataResults,
        );
      } else {
        // Original behavior - all devices together
        const deviceUuids = powerClampDevices.map((device) => device.uuid);
        const data = await this.executeProcedure(
          'fact_daily_space_energy_consumed_procedure',
          [formattedMonthDate, deviceUuids.join(',')],
        );

        // Format and filter data
        const formattedData = data.map((item) => ({
          ...item,
          date: new Date(item.date).toLocaleDateString('en-CA'), // YYYY-MM-DD
        }));

        const resultData = monthDate
          ? filterByMonth(formattedData, monthDate)
          : formattedData;

        return this.buildResponse(
          `Power clamp data fetched successfully for ${spaceUuid ? 'space' : 'community'}`,
          resultData,
        );
      }
    } catch (error) {
      console.error('Error fetching power clamp data', {
        error,
        spaceUuid,
        communityUuid,
      });
      throw new HttpException(
        error.response?.message || 'Failed to fetch power clamp data',
        error.status || HttpStatus.INTERNAL_SERVER_ERROR,
      );
    }
  }

  async getPowerClampData(
    params: PowerClampParamsDto,
    query: GetPowerClampDto,
  ) {
    const { dayDate, monthDate, year } = query;
    const { powerClampUuid } = params;
    try {
      const device = await this.deviceRepository.findOne({
        where: {
          uuid: powerClampUuid,
        },
      });
      if (!device) {
        throw new HttpException('Power clamp not found', HttpStatus.NOT_FOUND);
      }
      if (dayDate) {
        const data = await this.powerClampHourlyRepository
          .createQueryBuilder('hourly')
          .where('hourly.deviceUuid = :deviceUuid', {
            deviceUuid: powerClampUuid,
          })
          .andWhere('hourly.date = :date', { date: dayDate })
          .orderBy('CAST(hourly.hour AS INTEGER)', 'ASC')
          .getMany();

        return this.buildResponse(
          `Power clamp data for day ${dayDate} fetched successfully`,
          data,
        );
      }

      if (monthDate) {
        const data = await this.powerClampDailyRepository
          .createQueryBuilder('daily')
          .where('daily.deviceUuid = :deviceUuid', {
            deviceUuid: powerClampUuid,
          })
          .andWhere("TO_CHAR(daily.date, 'YYYY-MM') = :monthDate", {
            monthDate,
          })
          .orderBy('daily.date', 'ASC')
          .getMany();

        return this.buildResponse(
          `Power clamp data for month ${monthDate} fetched successfully`,
          data,
        );
      }

      if (year) {
        const data = await this.powerClampMonthlyRepository
          .createQueryBuilder('monthly')
          .where('monthly.deviceUuid = :deviceUuid', {
            deviceUuid: powerClampUuid,
          })
          .andWhere('RIGHT(monthly.month, 4) = :year', { year })
          .orderBy('monthly.month', 'ASC')
          .getMany();

        return this.buildResponse(
          `Power clamp data for year ${year} fetched successfully`,
          data,
        );
      }

      return this.buildResponse(`Power clamp data fetched successfully`, []);
    } catch (error) {
      throw new HttpException(
        error.message || 'Error fetching power clamp data',
        error.status || HttpStatus.INTERNAL_SERVER_ERROR,
      );
    }
  }

  private buildResponse(message: string, data: any[]) {
    return new SuccessResponseDto({
      message,
      data,
      statusCode: HttpStatus.OK,
    });
  }
  private async executeProcedure(
    procedureFileName: string,
    params: (string | number | null)[],
  ): Promise<any[]> {
    const query = this.loadQuery(
      'fact_space_energy_consumed',
      procedureFileName,
    );
    return await this.dataSource.query(query, params);
  }
  private loadQuery(folderName: string, fileName: string): string {
    return this.sqlLoader.loadQuery(folderName, fileName, SQL_PROCEDURES_PATH);
  }
}
