import { ORPHAN_SPACE_NAME } from '@app/common/constants/orphan-constant';
import { ApiProperty } from '@nestjs/swagger';
import { Type } from 'class-transformer';
import {
  ArrayUnique,
  IsArray,
  IsMongoId,
  IsNotEmpty,
  IsNumber,
  IsOptional,
  IsString,
  IsUUID,
  NotEquals,
  ValidateNested,
} from 'class-validator';
import { CreateProductAllocationDto } from './create-product-allocation.dto';
import { AddSubspaceDto } from './subspace';

export class AddSpaceDto {
  @ApiProperty({
    description: 'Name of the space (e.g., Floor 1, Unit 101)',
    example: 'Unit 101',
  })
  @IsString()
  @IsNotEmpty()
  @NotEquals(ORPHAN_SPACE_NAME, {
    message() {
      return `Space name cannot be "${ORPHAN_SPACE_NAME}". Please choose a different name.`;
    },
  })
  spaceName: string;

  @ApiProperty({
    description: 'UUID of the parent space (if any, for hierarchical spaces)',
    example: 'f5d7e9c3-44bc-4b12-88f1-1b3cda84752e',
    required: false,
  })
  @IsUUID()
  @IsOptional()
  parentUuid?: string;

  @ApiProperty({
    description: 'Icon identifier for the space',
    example: 'assets/location',
    required: false,
  })
  @IsString()
  @IsOptional()
  public icon?: string;

  @ApiProperty({ description: 'X position on canvas', example: 120 })
  @IsNumber()
  x: number;

  @ApiProperty({ description: 'Y position on canvas', example: 200 })
  @IsNumber()
  y: number;

  @ApiProperty({
    description: 'UUID of the Space Model',
    example: 'd290f1ee-6c54-4b01-90e6-d701748f0851',
  })
  @IsMongoId()
  @IsOptional()
  spaceModelUuid?: string;

  @ApiProperty({
    description: 'List of subspaces included in the model',
    type: [AddSubspaceDto],
  })
  @IsOptional()
  @IsArray()
  @ValidateNested({ each: true })
  @ArrayUnique((subspace) => subspace.subspaceName, {
    message(validationArguments) {
      const subspaces = validationArguments.value;
      const nameCounts = subspaces.reduce((acc, curr) => {
        acc[curr.subspaceName] = (acc[curr.subspaceName] || 0) + 1;
        return acc;
      }, {});
      // Find duplicates
      const duplicates = Object.keys(nameCounts).filter(
        (name) => nameCounts[name] > 1,
      );
      return `Duplicate subspace names found: ${duplicates.join(', ')}`;
    },
  })
  @Type(() => AddSubspaceDto)
  subspaces?: AddSubspaceDto[];

  @ApiProperty({
    description: 'List of allocations associated with the space',
    type: [CreateProductAllocationDto],
  })
  @IsOptional()
  @IsArray()
  @ValidateNested({ each: true })
  @Type(() => CreateProductAllocationDto)
  productAllocations?: CreateProductAllocationDto[];

  @ApiProperty({
    description: 'List of children spaces associated with the space',
    type: [AddSpaceDto],
  })
  @IsOptional()
  @ValidateNested({ each: true })
  @Type(() => AddSpaceDto)
  children?: AddSpaceDto[];
}
